// main.c


// Contains the .dll entry point and overridden functions
// uses imports.c for loading symbols, dib.c for getting framebuffer access.

// Copr. 1998 - Justin Frankel/Nullsoft, Inc. (http://www.nullsoft.com)

// Version 0.5 - 1/4/1998

#include <windows.h>

#define FX_GLIDE_NO_FUNC_PROTO
#include <glide.h>
#include <glideutl.h>

#include "pointers.h"

HINSTANCE hGlide, hDllInst;

#define ENTRY __declspec( dllexport )
#define CALL __stdcall

// dib.c functions
extern void dibinit(HWND hwnd, int w, int h);
extern void dibquit();
extern short *dibgetbuf();
extern void dibupdate();
extern int dibgetpitch();

// imports.c functions
extern int LoadSymbols(HINSTANCE *hGlide, HINSTANCE hDllInst);

// DLL entry point. Calls on load/unload of the .dll. Loads or frees the real glide2x.dll.
BOOL WINAPI dllmain(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
{
	hDllInst = hInst;
	if (ul_reason_for_call == DLL_PROCESS_ATTACH)
	{
		if (!LoadSymbols(&hGlide,hDllInst)) 
		{
			hGlide = NULL;
			return FALSE;
		}
	}
	if (ul_reason_for_call == DLL_PROCESS_DETACH)
	{
		FreeModule(hGlide);
		hGlide = NULL;
	}
	return TRUE;
}

int enable=1, // are we running windowed?
    sw, // screen width (window width)
	sh, // screen height (window height)
	offset_forlowresmodesinquake2; // quake 2 320x240 and 400x300 hack.


// Our new, overridden functions go here. 


// override grSstWinOpen, for opening the 3dfx hardware.
ENTRY FxBool CALL grSstWinOpen(
          FxU32                hWnd,
          GrScreenResolution_t screen_resolution,
          GrScreenRefresh_t    refresh_rate,
          GrColorFormat_t      color_format,
          GrOriginLocation_t   origin_location,
          int                  nColBuffers,
          int                  nAuxBuffers)
{
	HWND hwnd=(HWND)hWnd; // get hWnd as a HWND.

	if (_grSstWinOpen(hWnd,screen_resolution,refresh_rate,color_format,origin_location,nColBuffers,nAuxBuffers) == FXFALSE)
		return FXFALSE;

	sw=_grSstScreenWidth();
	sh=_grSstScreenHeight();
	
	// decide if we are running fullscreen or windowed
	if (hwnd && IsWindow(hwnd)) 
	{
		RECT r;
		GetWindowRect(hwnd,&r);
		if (r.left == 0 && r.top == 0 && r.right == sw && r.bottom == sh)
			enable=0;
		else enable = 1;
	} 
	else enable=0;

	
	if (enable) // if enabled, initialize us
	{
		RECT r;
		GetClientRect(hwnd,&r);
		if (origin_location == GR_ORIGIN_LOWER_LEFT && sh == 384) 
			offset_forlowresmodesinquake2=384-r.bottom;
		else offset_forlowresmodesinquake2=0;
		sw = r.right;
		sh = r.bottom;
		_grSstControl(GR_CONTROL_DEACTIVATE);
		_grRenderBuffer(GR_BUFFER_BACKBUFFER);
		dibinit(hwnd,sw,sh);
	}
	return FXTRUE;
}

// override grBufferSwap to copy the hardware's data into our DIBSection
ENTRY void CALL grBufferSwap( int swap_interval )
{
	if (enable)
	{
		short *data = dibgetbuf();
		if (data)
		{
			_grLfbReadRegion(GR_BUFFER_BACKBUFFER,0,offset_forlowresmodesinquake2,sw,sh,dibgetpitch(),data);
			dibupdate();
		}
	}
	else _grBufferSwap(swap_interval);
}

// override grSstWinClose to delete our DIB
ENTRY void CALL grSstWinClose( void )
{
	_grSstWinClose();
	if (enable) dibquit();
}

// override grSstControl to keep the calling app from reenabling the hardware's video output
ENTRY FxBool CALL grSstControl( FxU32 code )
{
	if (!enable) return _grSstControl(code);
	return 0;
}